-- ==============================================================
-- FILE : USER_AP4.VHD  -  MUST be modified by the USER
-- ==============================================================
--
-- This file is part of Example 4, for the HERON-IO5 module.
--
-- This file must be modified by the user.
--
-- ==============================================================
--
--  Date        : 08/04/2004
--  Author      : T Hollis - HUNT ENGINEERING
--  Description : User code section
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     T Hollis       08-04-04    First Written for the IO5
--  1.1     T. Hollis      08-03-05    Continuous data transfer at
--                                     160MHz, DAC_DCM_RES included,
--                                     DAC_SPI_RES included.
--  1.2     R. Williams    08-06-06    Removed HEPC8 support.
--                                     Minor fix to HE_RD_6F logic.
--
--  1.3     T. Holmboe		02-05-11		Clock source changed to AC-Clock
--													(CLKIN) which runs at 40 MHz. No need
--													to change the frequency, i.e.
--													no need for BUFG_DFS_SUBM
--
-- ==============================================================



-- ==============================================================
--   Package : CONFIG
-- ==============================================================
--
-- This package allow users to set up the different clock domains
-- of the application (FIFO, ADC interfaces) in the top level.
-- Please refer to the HUNT documentation for more details.
--
-- ==============================================================

package CONFIG is

  -- FIFO CLOCK
  -- ==========

--   --------------------------------------------------------------------
--   Normally you will use the same clock source for the input and output
--   HERON FIFOs. In that case you need a single clock domain.
--   Then you drive SRC_FCLK_G with the frequency you want, which for the 
--   HEPC9/HECPCI9/HERON_BASE2 must be >=60Mhz and <= 100Mhz.
--   The clock you use in your design is then FCLK_G which has the corrct 
--   phase relative to the FIFO data.
--   In this case set FCLK_G_DOMAIN = TRUE

--   If however you want to use different clocks for the input and output 
--   HERON FIFOs, you have seperate clock domains.
--   Now you drive the signals SRC_FCLK_RD and SRC_FCLK_WR with the frequencies
--   you want, (still probably >60Mhz and <100Mhz) and your design must use 
--   FCLK_RD to deal with input FIFO data but FCLK_WR to deal with output FIFO data.
--   In this case set FCLK_G_DOMAIN = FALSE

  -- To use a single common FIFO clock, set FCLK_G_DOMAIN = TRUE
  constant FCLK_G_DOMAIN : boolean := TRUE;

  -- ADC CLOCK
  -- =========

  -- To use a single common ADC clock for A & B channels, set SCLK_G_DOMAIN = TRUE
  constant SCLK_G_DOMAIN : boolean := TRUE;

  -- To drive both ADC sample clocks from a signal inside the FPGA set INTERNAL_SCLK_G = TRUE
  -- to use an external sample clock, set INTERNAL_SCLK_G = FALSE
  -- This constant is used only when SCLK_G_DOMAIN = TRUE
  -- constant INTERNAL_SCLK_G : boolean := TRUE;
  constant INTERNAL_SCLK_G : boolean := FALSE;  -- T.Holmboe

  -- To drive ADC sample clock A from a signal inside the FPGA set INTERNAL_SCLK_A = TRUE
  -- to use an external sample clock, set INTERNAL_SCLK_A = FALSE
  -- This constant is used only when SCLK_G_DOMAIN = FALSE
  constant INTERNAL_SCLK_A : boolean := TRUE;

  -- To drive ADC sample clock B from a signal inside the FPGA set INTERNAL_SCLK_B = TRUE
  -- to use an external sample clock, set INTERNAL_SCLK_B = FALSE
  -- This constant is used only when SCLK_G_DOMAIN = FALSE
  constant INTERNAL_SCLK_B : boolean := TRUE;

  -- To select the differential signal range for ADC A set DIFF_RANGE_A = TRUE
  -- to select the single-ended signal range for ADC A set DIFF_RANGE_A = FALSE
  constant DIFF_RANGE_A : boolean := FALSE;

  -- To select the differential signal range for ADC B set DIFF_RANGE_B = TRUE
  -- to select the single-ended signal range for ADC B set DIFF_RANGE_B = FALSE
  constant DIFF_RANGE_B : boolean := FALSE;

  -- DAC CLOCK
  -- =========

  -- To drive the DAC sample clocks from a signal inside the FPGA set INTERNAL_SCLK_DAC = TRUE
  -- to use an external sample clock, set INTERNAL_SCLK_DAC = FALSE
  -- constant INTERNAL_SCLK_DAC : boolean := TRUE;
  constant INTERNAL_SCLK_DAC : boolean := FALSE; -- T.Holmboe

  -- To use a configuration suitable for using the DAC PLL  set DAC_PLL = TRUE
  constant DAC_PLL : boolean := FALSE;

  -- Set DAC_FREQ to the frequency in MHz of the DAC Sample Clock
  constant DAC_FREQ : integer := 160;  -- Does not apply here. T.Holmboe

end CONFIG;


-- ==============================================================
--   Module : USER_AP
-- ==============================================================

library IEEE;
  use IEEE.std_logic_1164.ALL;
  use IEEE.numeric_std.all;
  use WORK.CONFIG.all;

--pragma translate_off
library sim_lib;
  use sim_lib.all;

library UNISIM;
  use UNISIM.VCOMPONENTS.all;
--pragma translate_on

entity USER_AP is
  port (
    RESET           : in  std_logic;                     -- asynchronous reset active high
    CONFIG          : in  std_logic;                     -- System wide Config signal
-- CLOCK IO
    OSC1            : in  std_logic;                     -- Xtal OSC1
    CLKIN           : in  std_logic;                     -- User clock input (unbuffered)
-- MISCELLANEOUS IO
    UDPRES          : out std_logic;                     -- reset the carrier (high if NOT used)
    LED             : out std_logic_vector(4 downto 0);  -- LEDs
    ADDR_FLAGSEL    : in  std_logic;
    BOOTEN          : out std_logic;
    CID             : in  std_logic_vector(3 downto 0);
    MID             : in  std_logic_vector(3 downto 0);
-- UMIs
    UMI_IN          : in  std_logic_vector(3 downto 0);
    UMI_OUT         : out std_logic_vector(3 downto 0);
    UMI_EN          : out std_logic_vector(3 downto 0);
-- DIGITAL IO
    DIO_IN          : in  std_logic_vector(7 downto 0);
    DIO_OUT         : out std_logic_vector(7 downto 0);
    DIO_EN          : out std_logic_vector(7 downto 0);
-- READ FIFO CLK
    FCLK_RD         : in  std_logic;                     -- FIFO_IN clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_RD     : out std_logic;                     -- FIFO_IN clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- WRITE FIFO CLK
    FCLK_WR         : in  std_logic;                     -- FIFO_OUT clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_WR     : out std_logic;                     -- FIFO_OUT clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- COMMON FIFO CLK
    FCLK_G          : in  std_logic;                     -- Common FIFO clock to be used in this module (only when FCLK_G_DOMAIN=TRUE)
    SRC_FCLK_G      : out std_logic;                     -- Common FIFO clock source for the top level  (only when FCLK_G_DOMAIN=TRUE)
  -- FIFO IN / HE_RD_6F interface
    INFIFO_READ_REQ : out std_logic_vector(5 downto 0);
    INFIFO_DVALID   : in  std_logic_vector(5 downto 0);
    INFIFO_SINGLE   : in  std_logic_vector(5 downto 0);
    INFIFO_BURST    : in  std_logic_vector(5 downto 0);
    INFIFO0_D       : in  std_logic_vector(31 downto 0);
    INFIFO1_D       : in  std_logic_vector(31 downto 0);
    INFIFO2_D       : in  std_logic_vector(31 downto 0);
    INFIFO3_D       : in  std_logic_vector(31 downto 0);
    INFIFO4_D       : in  std_logic_vector(31 downto 0);
    INFIFO5_D       : in  std_logic_vector(31 downto 0);
  -- FIFO OUT / HE_WR_6F interface
    OUTFIFO_READY   : in  std_logic_vector(5 downto 0);
    OUTFIFO_WRITE   : out std_logic_vector(5 downto 0);
    OUTFIFO_D       : out std_logic_vector(31 downto 0);
-- SERIAL BUS / HE_USER Message interface
    MSG_CLK         : out std_logic;                     -- HSB clock source for the top level
    MSG_DIN         : in  std_logic_vector(7 downto 0);  -- data received from HSB
    MSG_ADDR        : in  std_logic_vector(7 downto 0);  -- HSB address
    MSG_AEN         : in  std_logic;                     -- address enable
    MSG_WEN         : in  std_logic;                     -- write access
    MSG_REN         : in  std_logic;                     -- read access
    MSG_DONE        : in  std_logic;                     -- message successfully transmitted
    MSG_COUNT       : in  std_logic;                     -- user's counter enable
    MSG_CLEAR       : in  std_logic;                     -- user's counter clear
    MSG_READY       : out std_logic;                     -- to acknowledge an access
    MSG_SEND_MSG    : out std_logic;                     -- message send command
    MSG_CE          : out std_logic;                     -- to control speed operation
    MSG_DOUT        : out std_logic_vector(7 downto 0);  -- data sent to HSB
    MSG_SEND_ID     : out std_logic;                     -- ID information send command
    MSG_LAST_BYTE   : out std_logic;                     -- to indicate when the last byte is presented
-- CHANNEL A SCLK
    ADC_DCO_A       : in  std_logic;                     -- ADC data clock A 
    SRC_SCLK_A      : out std_logic;                     -- ADC clock source A for the top level (only used when INTERNAL_SCLK_A=TRUE)
    DATA_SYNC_A     : out std_logic;                     -- ADC data sync A
-- CHANNEL B SCLK
    ADC_DCO_B       : in  std_logic;                     -- ADC data clock B 
    SRC_SCLK_B      : out std_logic;                     -- ADC clock source B for the top level (only used when INTERNAL_SCLK_B=TRUE)
    DATA_SYNC_B     : out std_logic;                     -- ADC data sync B
-- GLOBAL SCLK
    SRC_SCLK_G      : out std_logic;
-- HE_ADC
    ADC_A_A         : in  std_logic_vector(11 downto 0); -- ADC A data bus A
    ADC_A_B         : in  std_logic_vector(11 downto 0); -- ADC A data bus B
    ADC_B_A         : in  std_logic_vector(11 downto 0); -- ADC B data bus A
    ADC_B_B         : in  std_logic_vector(11 downto 0); -- ADC B data bus B
    OTR_A_A         : in  std_logic;                     -- ADC A over range flag A
    OTR_A_B         : in  std_logic;                     -- ADC A over range flag B
    OTR_B_A         : in  std_logic;                     -- ADC B over range flag A
    OTR_B_B         : in  std_logic;                     -- ADC B over range flag B
-- HE_DAC
    SRC_DAC_SCLK    : out std_logic;                     -- DAC clock source for the top level
    DAC_SCLK        : in  std_logic;                     -- DAC sample clock
    DAC_A           : out std_logic_vector(15 downto 0); -- DAC data bus A
    DAC_B           : out std_logic_vector(15 downto 0); -- DAC data bus B
    DAC_DCM_RES     : out std_logic;                     -- Reset for DAC DCM
    DAC_SPI_RES     : out std_logic;                     -- Reset for DAC SPI
-- DAC Serial Interface
    SPI_CLOCK       : out std_logic;                     -- DAC Serial Interface System Clock
    SPI_R_W         : out std_logic;                     -- DAC Serial Interface Read/Write
    SPI_COUNT       : out std_logic_vector(2 downto 0);  -- DAC Serial Interface Count
    SPI_ADDRESS     : out std_logic_vector(4 downto 0);  -- DAC Serial Interface Address
    SPI_WR_DATA     : out std_logic_vector(7 downto 0);  -- DAC Serial Interface Write Data
    SPI_LOAD_0      : out std_logic;                     -- DAC Serial Interface Load 0
    SPI_LOAD_1      : out std_logic;                     -- DAC Serial Interface Load 1
    SPI_LOAD_2      : out std_logic;                     -- DAC Serial Interface Load 2
    SPI_LOAD_3      : out std_logic;                     -- DAC Serial Interface Load 3
    SPI_TRANSMIT    : out std_logic;                     -- DAC Serial Interface Transmit
    SPI_ACTIVE      : in  std_logic;                     -- DAC Serial Interface Active
    SPI_RD_DATA_0   : in  std_logic_vector(7 downto 0);  -- DAC Serial Interface Read Data 0
    SPI_RD_DATA_1   : in  std_logic_vector(7 downto 0);  -- DAC Serial Interface Read Data 1
    SPI_RD_DATA_2   : in  std_logic_vector(7 downto 0);  -- DAC Serial Interface Read Data 2
    SPI_RD_DATA_3   : in  std_logic_vector(7 downto 0)   -- DAC Serial Interface Read Data 3
  );
end USER_AP;


architecture EXAMPLE4 of USER_AP is

  component BUFG 
    port (
      I : in std_logic;
      O : out std_logic
    ); 
  end component;

  component adc_2x_dcm
    port (
      CLK_IN : in  std_logic;
      RST    : in  std_logic;
      CLK2X  : out std_logic;
      LOCK   : out std_logic
    );
  end component;

  component BUFG_DFS_SUBM is
    port ( 
      CLK_IN : in  std_logic;
      RST    : in  std_logic;
      CLKFX  : out std_logic;
      LOCK   : out std_logic
    );
  end component;

  -- Asynchronous fifo / generated by Xilinx CoreGEN
  -- Memory type = block, Input data width = 32, FIFO depth = 511
  component fifo31x16
    port (
      din         : in  std_logic_vector(15 downto 0);
      wr_en       : in  std_logic;
      wr_clk      : in  std_logic;
      rd_en       : in  std_logic;
      rd_clk      : in  std_logic;
      ainit       : in  std_logic;
      dout        : out std_logic_vector(15 downto 0);
      full        : out std_logic;
      empty       : out std_logic;
      almost_full : out std_logic
    );
  end component;

  signal VCC           : std_logic;
  signal MCLK          : std_logic;  -- Main clock
  signal LEDcount      : unsigned(25 downto 0);
  signal LEDcount2     : unsigned(25 downto 0);  -- T.holmboe

  signal DFS_OP        : std_logic;
  signal DFS_LOCK      : std_logic;
  signal ADC_DCO_A_2X  : std_logic;
  signal ADC_DCO_B_2X  : std_logic;
  signal LOCK_A        : std_logic;
  signal LOCK_B        : std_logic;
  signal MUX_A         : std_logic;
  signal MUX_B         : std_logic;
  signal MUXR_A        : std_logic;
  signal MUXR_B        : std_logic;
  signal ADC_A         : std_logic_vector(11 downto 0);
  signal ADC_B         : std_logic_vector(11 downto 0);
  signal DATA_A        : std_logic_vector(15 downto 0);
  signal DATA_B        : std_logic_vector(15 downto 0);
  signal IN_DATA_A     : std_logic_vector(15 downto 0);
  signal IN_DATA_B     : std_logic_vector(15 downto 0);
  signal FF_REN_A      : std_logic;
  signal FF_REN_B      : std_logic;
  signal FF_EMPTY_A    : std_logic;
  signal FF_EMPTY_B    : std_logic;
  signal FF_RESET      : std_logic;

  signal DFS_RESET     : std_logic;
  signal RESR1         : std_logic;
  signal RESR2         : std_logic;
  signal RESR3         : std_logic;
  signal RESR4         : std_logic;
  signal RST_2X        : std_logic;

begin

  -- ==========================================================
  -- ==========================================================
  --       <<<<<<<   INSERT YOUR CODE HERE    >>>>>>>>>
  -- ==========================================================
  -- ==========================================================

  VCC <= '1';

  -------------------------------------------------------------
  -- OSC1 clock

  MCLK <= OSC1;

  -------------------------------------------------------------
  -- Drive the UDP-RESET high (inactive)

  UDPRES <= '1';

  -------------------------------------------------------------
  -- Drive the BOOT ENABLE signal high (inactive)

  BOOTEN <= '1';

  -------------------------------------------------------------
  -- Drive unused signals

  DIO_OUT(7 downto 0)   <= (others=>'0');
  DIO_EN(7 downto 0)    <= (others=>'1');

  UMI_OUT   <= (others=>'0');
  UMI_EN    <= (others=>'1');

  SPI_CLOCK    <= '0';
  SPI_R_W      <= '0';
  SPI_COUNT    <= (others=>'0');
  SPI_ADDRESS  <= (others=>'0');
  SPI_WR_DATA  <= (others=>'0');
  SPI_LOAD_0   <= '0';
  SPI_LOAD_1   <= '0';
  SPI_LOAD_2   <= '0';
  SPI_LOAD_3   <= '0';
  SPI_TRANSMIT <= '0';

  -------------------------------------------------------------
  -- IN/OUT FIFO clocks

  SRC_FCLK_RD <= '0';   -- clock source for READ FIFO
  SRC_FCLK_WR <= '0';   -- clock source for WRITE FIFO
  SRC_FCLK_G  <= MCLK;  -- global clock source for both FIFOs

  INFIFO_READ_REQ <= (others=>'0');
  OUTFIFO_WRITE   <= (others=>'0');

  OUTFIFO_D   <= (others=>'0');

  -------------------------------------------------------------
  -- Serial Bus

  MSG_CLK <= OSC1;  -- clock for Message Interface

  MSG_READY     <= '1';
  MSG_SEND_MSG  <= '0';
  MSG_CE        <= '0';
  MSG_SEND_ID   <= '0';
  MSG_LAST_BYTE <= '0';
  MSG_DOUT      <= (others=>'0');

  -------------------------------------------------------------
  -- ADC

  SRC_SCLK_A <= '0';   -- clock source for ADC_A
  SRC_SCLK_B <= '0';   -- clock source for ADC_B
  --SRC_SCLK_G <= OSC1;  -- global clock source for both ADCs

  --FREQUENCY SYNTHESISER TO GENERATE 160MHz FROM 100MHz
  iDFS : BUFG_DFS_SUBM 
  port map ( 
     CLK_IN    => OSC1,
     RST       => RESET,
     CLKFX     => DFS_OP,
     LOCK      => DFS_LOCK
       );
  SRC_SCLK_G <= DFS_OP; --160MHz


--THE 2X_DCM'S MUST BE RESET AFTER 160MHz DFS IS LOCKED
  DFS_RESET <= RESET or (not DFS_LOCK);
--AND EXTRA DELAY
      process(OSC1)
    begin
      if rising_edge(OSC1) then
	   RESR1 <= DFS_RESET;
	   RESR2 <= RESR1;
	   RESR3 <= RESR2;
	   RESR4 <= RESR3;
        RST_2X <= (RESR2 or RESR3 or RESR4 or RESR1);
      end if;
    end process;

  DATA_SYNC_A <= '0';
  DATA_SYNC_B <= '0';

--FREQUENCY DOUBLERS
iadc_2x_dcm_a : adc_2x_dcm
    port map (
        CLK_IN   => ADC_DCO_A,
        RST      => RST_2X,
        CLK2X    => ADC_DCO_A_2X,
        LOCK     => LOCK_A );

iadc_2x_dcm_b : adc_2x_dcm
    port map (
        CLK_IN   => ADC_DCO_B,
        RST      => RST_2X,
        CLK2X    => ADC_DCO_B_2X,
        LOCK     => LOCK_B );

-- HOLD THE FIFO's IN RESET UNTIL ALL LOCKED
 process(ADC_DCO_A_2X)
  begin

	if rising_edge(ADC_DCO_A_2X) then
	 FF_RESET <= RESET or (not (DFS_LOCK and LOCK_A and LOCK_B));
	end if;

  end process;

--MULTIPLEX THE DATA FOR ADC A ONTO SINGLE BUS
  process(ADC_DCO_A_2X)
  begin
   
      if falling_edge(ADC_DCO_A_2X) then
	   MUX_A <= ADC_DCO_A;
	 end if;

	 if rising_edge(ADC_DCO_A_2X) then
	   MUXR_A <= MUX_A;

	   	     if MUXR_A = '0' then
			  ADC_A <= ADC_A_A;
			else
			  ADC_A <= ADC_A_B;
			end if;

  	 end if;
	 
  end process;



  --MULTIPLEX THE DATA FOR ADC B ONTO SINGLE BUS
  process(ADC_DCO_B_2X)
  begin
   
      if falling_edge(ADC_DCO_B_2X) then
	   MUX_B <= ADC_DCO_B;
      end if;

	 if rising_edge(ADC_DCO_B_2X) then
	   MUXR_B <= MUX_B;

  	          if MUXR_B = '0' then
	             ADC_B <= ADC_B_A;
	          else
	             ADC_B <= ADC_B_B;
               end if;

  	 end if;     	  

  end process;



  IN_DATA_A <= ADC_A & "0000";
  IN_DATA_B <= ADC_B & "0000";

   iIN_FIFO_A : fifo31x16
    port map (
      din          => IN_DATA_A,
      wr_en        => '1',
      wr_clk       => ADC_DCO_A_2X,
      rd_en        => FF_REN_A,
      rd_clk       => DAC_SCLK,
      ainit        => FF_RESET,
      dout         => DATA_A,
      full         => open,
      empty        => FF_EMPTY_A,
      almost_full  => open
	  );

  iIN_FIFO_B : fifo31x16
    port map (
      din          => IN_DATA_B,
      wr_en        => '1',
      wr_clk       => ADC_DCO_B_2X,
      rd_en        => FF_REN_B,
      rd_clk       => DAC_SCLK,
      ainit        => FF_RESET,
      dout         => DATA_B,
      full         => open,
      empty        => FF_EMPTY_B,
      almost_full  => open
	  );

  FF_REN_A <= (NOT FF_EMPTY_A);
  FF_REN_B <= (NOT FF_EMPTY_B);

  

  -------------------------------------------------------------
  -- DAC

  -- DAC Clock source
   SRC_DAC_SCLK <= DFS_OP; --160MHz

  DAC_A <= DATA_A;
  DAC_B <= DATA_B;

-- RESET FOR DAC DCM IN HIL
--  DAC_DCM_RES <= RESET;
  DAC_DCM_RES <= FF_RESET or RESET;

-- RESET FOR DAC SERIAL INTERFACE
  DAC_SPI_RES <= '0';

  -------------------------------------------------------------
  -- Led Flasher : dividing F_CLKG clock by 2**25

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      LEDcount <= (others=>'0');
    elsif rising_edge(FCLK_G) then
      LEDcount <= LEDcount + 1;
    end if;
  end process;
  
  -------------------------------------------------------------
  -- Led Flasher2 : dividing F_CLKG clock by 2**25  T.Holmboe

  process(RESET, ADC_DCO_A_2X)
  begin
    if RESET='1' then
      LEDcount2 <= (others=>'0');
    elsif rising_edge(ADC_DCO_A_2X) then
      LEDcount2 <= LEDcount2 + 1;
    end if;
  end process;
  -------------------------------------------------------------

  LED(4) <= LEDcount(24);
  LED(3) <= NOT DFS_LOCK;
  LED(2) <= NOT LOCK_A;
  LED(1) <= NOT LOCK_B;
  -- Drive LEDs high (inactive)
  -- LED(0) <= '1'; T.Holmboe
  LED(0) <= LEDcount2(24);  -- T.Holmboe

end EXAMPLE4;
